@@ -1,19 +1,22 @@
 import logging
 import os
+import time
 import traceback
 from math import floor
-from typing import List, Tuple
+from threading import Thread
+from typing import List, Tuple, Optional, Dict
 
 from PyQt5.QtWidgets import QApplication, QStyleFactory
 
-from bauh import ROOT_DIR
+from bauh import ROOT_DIR, __app_name__
 from bauh.api.abstract.controller import SoftwareManager
 from bauh.api.abstract.download import FileDownloader
 from bauh.api.abstract.view import ViewComponent, TabComponent, InputOption, TextComponent, MultipleSelectComponent, \
     PanelComponent, FormComponent, TabGroupComponent, SingleSelectComponent, SelectViewType, TextInputComponent, \
     FileChooserComponent, RangeInputComponent
-from bauh.view.core import config, timeshift
-from bauh.view.core.config import read_config
+from bauh.commons.view_utils import new_select
+from bauh.view.core import timeshift
+from bauh.view.core.config import CoreConfigManager
 from bauh.view.core.downloader import AdaptableFileDownloader
 from bauh.view.util import translation
 from bauh.view.util.translation import I18n
@@ -22,12 +25,13 @@ from bauh.view.util.translation import I18n
 class GenericSettingsManager:
 
     def __init__(self, managers: List[SoftwareManager], working_managers: List[SoftwareManager],
-                 logger: logging.Logger, i18n: I18n, file_downloader: FileDownloader):
+                 logger: logging.Logger, i18n: I18n, file_downloader: FileDownloader, configman: CoreConfigManager):
         self.i18n = i18n
         self.managers = managers
         self.working_managers = working_managers
         self.logger = logger
         self.file_downloader = file_downloader
+        self.configman = configman
 
     def get_settings(self, screen_width: int, screen_height: int) -> ViewComponent:
         tabs = list()
@@ -53,7 +57,7 @@ class GenericSettingsManager:
                 if man.is_enabled() and man in self.working_managers:
                     def_gem_opts.add(opt)
 
-        core_config = read_config()
+        core_config = self.configman.get_config()
 
         if gem_opts:
             type_help = TextComponent(html=self.i18n['core.config.types.tip'])
@@ -100,14 +104,14 @@ class GenericSettingsManager:
                                             max_width=default_width,
                                             id_="icon_exp")
 
-        select_trim_up = self._gen_select(label=self.i18n['core.config.trim.after_upgrade'],
-                                          tip=self.i18n['core.config.trim.after_upgrade.tip'],
-                                          value=core_config['disk']['trim']['after_upgrade'],
-                                          max_width=default_width,
-                                          opts=[(self.i18n['yes'].capitalize(), True, None),
-                                                (self.i18n['no'].capitalize(), False, None),
-                                                (self.i18n['ask'].capitalize(), None, None)],
-                                          id_='trim_after_upgrade')
+        select_trim_up = new_select(label=self.i18n['core.config.trim.after_upgrade'],
+                                    tip=self.i18n['core.config.trim.after_upgrade.tip'],
+                                    value=core_config['disk']['trim']['after_upgrade'],
+                                    max_width=default_width,
+                                    opts=[(self.i18n['yes'].capitalize(), True, None),
+                                          (self.i18n['no'].capitalize(), False, None),
+                                          (self.i18n['ask'].capitalize(), None, None)],
+                                    id_='trim_after_upgrade')
 
         select_dep_check = self._gen_bool_component(label=self.i18n['core.config.system.dep_checking'],
                                                     tooltip=self.i18n['core.config.system.dep_checking.tip'],
@@ -141,7 +145,7 @@ class GenericSettingsManager:
         if current_mthread_client not in available_mthread_clients:
             current_mthread_client = None
 
-        return self._gen_select(label=self.i18n['core.config.download.multithreaded_client'],
+        return new_select(label=self.i18n['core.config.download.multithreaded_client'],
                                 tip=self.i18n['core.config.download.multithreaded_client.tip'],
                                 id_="mthread_client",
                                 max_width=default_width,
@@ -160,8 +164,8 @@ class GenericSettingsManager:
 
         allowed_exts = {'png', 'svg', 'jpg', 'jpeg', 'ico', 'xpm'}
         select_def_icon = FileChooserComponent(id_='def_icon',
-                                               label=self.i18n["core.config.ui.tray.default_icon"].capitalize(),
-                                               tooltip=self.i18n["core.config.ui.tray.default_icon.tip"].capitalize(),
+                                               label=self.i18n["core.config.ui.tray.default_icon"],
+                                               tooltip=self.i18n["core.config.ui.tray.default_icon.tip"],
                                                file_path=str(core_config['ui']['tray']['default_icon']) if core_config['ui']['tray']['default_icon'] else None,
                                                max_width=default_width,
                                                allowed_extensions=allowed_exts)
@@ -177,7 +181,7 @@ class GenericSettingsManager:
         return TabComponent(self.i18n['core.config.tab.tray'].capitalize(), PanelComponent(sub_comps), None, 'core.tray')
 
     def _gen_ui_settings(self, core_config: dict, screen_width: int, screen_height: int) -> TabComponent:
-        default_width = floor(0.11 * screen_width)
+        default_width = floor(0.15 * screen_width)
 
         select_hdpi = self._gen_bool_component(label=self.i18n['core.config.ui.hdpi'],
                                                tooltip=self.i18n['core.config.ui.hdpi.tip'],
@@ -204,9 +208,9 @@ class GenericSettingsManager:
                                            min_value=100, max_value=400, step_value=5, value=scale * 100,
                                            max_width=default_width)
 
-        cur_style = QApplication.instance().style().objectName().lower() if not core_config['ui']['style'] else core_config['ui']['style']
-        style_opts = [InputOption(label=self.i18n['core.config.ui.style.default'].capitalize(), value=None)]
-        style_opts.extend([InputOption(label=s.capitalize(), value=s.lower()) for s in QStyleFactory.keys()])
+        cur_style = QApplication.instance().property('qt_style') if not core_config['ui']['qt_style'] else core_config['ui']['qt_style']
+
+        style_opts = [InputOption(label=s.capitalize(), value=s.lower()) for s in QStyleFactory.keys()]
 
         default_style = [o for o in style_opts if o.value == cur_style]
 
@@ -220,14 +224,21 @@ class GenericSettingsManager:
             default_style = default_style[0]
 
         select_style = SingleSelectComponent(label=self.i18n['style'].capitalize(),
+                                             tooltip=self.i18n['core.config.ui.qt_style.tooltip'],
                                              options=style_opts,
                                              default_option=default_style,
                                              type_=SelectViewType.COMBO,
                                              max_width=default_width,
                                              id_="style")
 
-        input_maxd = TextInputComponent(label=self.i18n['core.config.ui.max_displayed'].capitalize(),
-                                        tooltip=self.i18n['core.config.ui.max_displayed.tip'].capitalize(),
+        select_system_theme = self._gen_bool_component(label=self.i18n['core.config.ui.system_theme'],
+                                                       tooltip=self.i18n['core.config.ui.system_theme.tip'].format(app=__app_name__),
+                                                       value=bool(core_config['ui']['system_theme']),
+                                                       max_width=default_width,
+                                                       id_='system_theme')
+
+        input_maxd = TextInputComponent(label=self.i18n['core.config.ui.max_displayed'],
+                                        tooltip=self.i18n['core.config.ui.max_displayed.tip'],
                                         only_int=True,
                                         id_="table_max",
                                         max_width=default_width,
@@ -239,11 +250,14 @@ class GenericSettingsManager:
                                                  max_width=default_width,
                                                  value=core_config['download']['icons'])
 
-        sub_comps = [FormComponent([select_hdpi, select_ascale, select_scale, select_dicons, select_style, input_maxd], spaces=False)]
+        sub_comps = [FormComponent([select_hdpi, select_ascale, select_scale,
+                                    select_dicons, select_system_theme,
+                                    select_style, input_maxd], spaces=False)]
+
         return TabComponent(self.i18n['core.config.tab.ui'].capitalize(), PanelComponent(sub_comps), None, 'core.ui')
 
     def _gen_general_settings(self, core_config: dict, screen_width: int, screen_height: int) -> TabComponent:
-        default_width = floor(0.11 * screen_width)
+        default_width = floor(0.15 * screen_width)
 
         locale_opts = [InputOption(label=self.i18n['locale.{}'.format(k)].capitalize(), value=k) for k in translation.get_available_keys()]
 
@@ -280,6 +294,12 @@ class GenericSettingsManager:
                                                     max_width=default_width,
                                                     id_="sys_notify")
 
+        select_load_apps = self._gen_bool_component(label=self.i18n['core.config.boot.load_apps'],
+                                                    tooltip=self.i18n['core.config.boot.load_apps.tip'],
+                                                    value=bool(core_config['boot']['load_apps']),
+                                                    id_='boot.load_apps',
+                                                    max_width=default_width)
+
         select_sugs = self._gen_bool_component(label=self.i18n['core.config.suggestions.activated'].capitalize(),
                                                tooltip=self.i18n['core.config.suggestions.activated.tip'].capitalize(),
                                                id_="sugs_enabled",
@@ -293,14 +313,15 @@ class GenericSettingsManager:
                                       max_width=default_width,
                                       id_="sugs_by_type")
 
-        inp_reboot = self._gen_select(label=self.i18n['core.config.updates.reboot'],
-                                      tip=self.i18n['core.config.updates.reboot.tip'],
-                                      id_='ask_for_reboot',
-                                      max_width=default_width,
-                                      value=bool(core_config['updates']['ask_for_reboot']),
-                                      opts=[(self.i18n['ask'].capitalize(), True, None), (self.i18n['no'].capitalize(), False, None)])
+        inp_reboot = new_select(label=self.i18n['core.config.updates.reboot'],
+                                tip=self.i18n['core.config.updates.reboot.tip'],
+                                id_='ask_for_reboot',
+                                max_width=default_width,
+                                value=bool(core_config['updates']['ask_for_reboot']),
+                                opts=[(self.i18n['ask'].capitalize(), True, None),
+                                      (self.i18n['no'].capitalize(), False, None)])
 
-        sub_comps = [FormComponent([select_locale, select_store_pwd, select_sysnotify, select_sugs, inp_sugs, inp_reboot], spaces=False)]
+        sub_comps = [FormComponent([select_locale, select_store_pwd, select_sysnotify, select_load_apps, select_sugs, inp_sugs, inp_reboot], spaces=False)]
         return TabComponent(self.i18n['core.config.tab.general'].capitalize(), PanelComponent(sub_comps), None, 'core.gen')
 
     def _gen_bool_component(self, label: str, tooltip: str, value: bool, id_: str, max_width: int = 200) -> SingleSelectComponent:
@@ -321,8 +342,8 @@ class GenericSettingsManager:
                        backup: PanelComponent,
                        ui: PanelComponent,
                        tray: PanelComponent,
-                       gems_panel: PanelComponent) -> Tuple[bool, List[str]]:
-        core_config = config.read_config()
+                       gems_panel: PanelComponent) -> Tuple[bool, Optional[List[str]]]:
+        core_config = self.configman.get_config()
 
         # general
         general_form = general.components[0]
@@ -340,6 +361,7 @@ class GenericSettingsManager:
         core_config['suggestions']['by_type'] = sugs_by_type
 
         core_config['updates']['ask_for_reboot'] = general_form.get_component('ask_for_reboot').get_selected()
+        core_config['boot']['load_apps'] = general_form.get_component('boot.load_apps').get_selected()
 
         # advanced
         adv_form = advanced.components[0]
@@ -406,9 +428,12 @@ class GenericSettingsManager:
 
         style = ui_form.get_component('style').get_selected()
 
-        cur_style = core_config['ui']['style'] if core_config['ui']['style'] else QApplication.instance().style().objectName().lower()
+        cur_style = core_config['ui']['qt_style'] if core_config['ui']['qt_style'] else QApplication.instance().property('qt_style')
         if style != cur_style:
-            core_config['ui']['style'] = style
+            core_config['ui']['qt_style'] = style
+            QApplication.instance().setProperty('qt_style', style)
+
+        core_config['ui']['system_theme'] = ui_form.get_component('system_theme').get_selected()
 
         # gems
         checked_gems = gems_panel.components[1].get_component('gems').get_selected_values()
@@ -421,28 +446,55 @@ class GenericSettingsManager:
         core_config['gems'] = None if core_config['gems'] is None and len(checked_gems) == len(self.managers) else checked_gems
 
         try:
-            config.save(core_config)
+            self.configman.save_config(core_config)
             return True, None
         except:
             return False, [traceback.format_exc()]
 
+    def _save_manager_settings(self, man: SoftwareManager, panel: ViewComponent, success_map: Dict[str, bool], warnings: List[str]):
+        success = False
+
+        try:
+            res = man.save_settings(panel)
+
+            if res:
+                success, errors = res[0], res[1]
+
+                if errors:
+                    warnings.extend(errors)
+        except:
+            self.logger.error("An exception happened while {} was trying to save its settings".format(man.__class__.__name__))
+            traceback.print_exc()
+        finally:
+            success_map[man.__class__.__name__] = success
+
+    def _save_core_settings(self, root_component: TabGroupComponent, success_map: Dict[str, bool], warnings: List[str]):
+        success = False
+
+        try:
+            bkp = root_component.get_tab('core.bkp')
+            success, errors = self._save_settings(general=root_component.get_tab('core.gen').content,
+                                                  advanced=root_component.get_tab('core.adv').content,
+                                                  tray=root_component.get_tab('core.tray').content,
+                                                  backup=bkp.content if bkp else None,
+                                                  ui=root_component.get_tab('core.ui').content,
+                                                  gems_panel=root_component.get_tab('core.types').content)
+            if errors:
+                warnings.extend(errors)
+
+        except:
+            self.logger.error("An exception happened while saving the core settings")
+            traceback.print_exc()
+        finally:
+            success_map[self.__class__.__name__] = success
+
     def save_settings(self, component: TabGroupComponent) -> Tuple[bool, List[str]]:
+        ti = time.time()
+        save_threads, warnings, success_map = [], [], {}
 
-        saved, warnings = True, []
-
-        bkp = component.get_tab('core.bkp')
-        success, errors = self._save_settings(general=component.get_tab('core.gen').content,
-                                              advanced=component.get_tab('core.adv').content,
-                                              tray=component.get_tab('core.tray').content,
-                                              backup=bkp.content if bkp else None,
-                                              ui=component.get_tab('core.ui').content,
-                                              gems_panel=component.get_tab('core.types').content)
-
-        if not success:
-            saved = False
-
-        if errors:
-            warnings.extend(errors)
+        save_core = Thread(target=self._save_core_settings, args=(component, success_map, warnings), daemon=True)
+        save_core.start()
+        save_threads.append(save_core)
 
         for man in self.managers:
             if man:
@@ -452,18 +504,18 @@ class GenericSettingsManager:
                 if not tab:
                     self.logger.warning("Tab for {} was not found".format(man.__class__.__name__))
                 else:
-                    res = man.save_settings(tab.content)
+                    save_man = Thread(target=self._save_manager_settings(man, tab.content, success_map, warnings), daemon=True)
+                    save_man.start()
+                    save_threads.append(save_man)
 
-                    if res:
-                        success, errors = res[0], res[1]
+        for t in save_threads:
+            t.join()
 
-                        if not success:
-                            saved = False
+        success = all(success_map.values())
 
-                        if errors:
-                            warnings.extend(errors)
-
-        return saved, warnings
+        tf = time.time()
+        self.logger.info("Saving all settings took {0:.8f} seconds".format(tf - ti))
+        return success, warnings
 
     def _gen_backup_settings(self, core_config: dict, screen_width: int, screen_height: int) -> TabComponent:
         if timeshift.is_available():
@@ -479,64 +531,51 @@ class GenericSettingsManager:
                         (self.i18n['no'].capitalize(), False, None),
                         (self.i18n['ask'].capitalize(), None, None)]
 
-            install_mode = self._gen_select(label=self.i18n['core.config.backup.install'],
-                                            tip=None,
-                                            value=core_config['backup']['install'],
-                                            opts=ops_opts,
-                                            max_width=default_width,
-                                            id_='install')
+            install_mode = new_select(label=self.i18n['core.config.backup.install'],
+                                      tip=None,
+                                      value=core_config['backup']['install'],
+                                      opts=ops_opts,
+                                      max_width=default_width,
+                                      id_='install')
 
-            uninstall_mode = self._gen_select(label=self.i18n['core.config.backup.uninstall'],
-                                              tip=None,
-                                              value=core_config['backup']['uninstall'],
-                                              opts=ops_opts,
-                                              max_width=default_width,
-                                              id_='uninstall')
+            uninstall_mode = new_select(label=self.i18n['core.config.backup.uninstall'],
+                                        tip=None,
+                                        value=core_config['backup']['uninstall'],
+                                        opts=ops_opts,
+                                        max_width=default_width,
+                                        id_='uninstall')
 
-            upgrade_mode = self._gen_select(label=self.i18n['core.config.backup.upgrade'],
-                                            tip=None,
-                                            value=core_config['backup']['upgrade'],
-                                            opts=ops_opts,
-                                            max_width=default_width,
-                                            id_='upgrade')
+            upgrade_mode = new_select(label=self.i18n['core.config.backup.upgrade'],
+                                      tip=None,
+                                      value=core_config['backup']['upgrade'],
+                                      opts=ops_opts,
+                                      max_width=default_width,
+                                      id_='upgrade')
 
-            downgrade_mode = self._gen_select(label=self.i18n['core.config.backup.downgrade'],
-                                              tip=None,
-                                              value=core_config['backup']['downgrade'],
-                                              opts=ops_opts,
-                                              max_width=default_width,
-                                              id_='downgrade')
+            downgrade_mode = new_select(label=self.i18n['core.config.backup.downgrade'],
+                                        tip=None,
+                                        value=core_config['backup']['downgrade'],
+                                        opts=ops_opts,
+                                        max_width=default_width,
+                                        id_='downgrade')
 
-            mode = self._gen_select(label=self.i18n['core.config.backup.mode'],
-                                    tip=None,
-                                    value=core_config['backup']['mode'],
-                                    opts=[
-                                        (self.i18n['core.config.backup.mode.incremental'], 'incremental',
-                                         self.i18n['core.config.backup.mode.incremental.tip']),
-                                        (self.i18n['core.config.backup.mode.only_one'], 'only_one',
-                                         self.i18n['core.config.backup.mode.only_one.tip'])
-                                    ],
-                                    max_width=default_width,
-                                    id_='mode')
-            type_ = self._gen_select(label=self.i18n['type'].capitalize(),
-                                     tip=None,
-                                     value=core_config['backup']['type'],
-                                     opts=[('rsync', 'rsync', None), ('btrfs', 'btrfs', None)],
-                                     max_width=default_width,
-                                     id_='type')
+            mode = new_select(label=self.i18n['core.config.backup.mode'],
+                              tip=None,
+                              value=core_config['backup']['mode'],
+                              opts=[
+                                  (self.i18n['core.config.backup.mode.incremental'], 'incremental',
+                                   self.i18n['core.config.backup.mode.incremental.tip']),
+                                  (self.i18n['core.config.backup.mode.only_one'], 'only_one',
+                                   self.i18n['core.config.backup.mode.only_one.tip'])
+                              ],
+                              max_width=default_width,
+                              id_='mode')
+            type_ = new_select(label=self.i18n['type'].capitalize(),
+                               tip=None,
+                               value=core_config['backup']['type'],
+                               opts=[('rsync', 'rsync', None), ('btrfs', 'btrfs', None)],
+                               max_width=default_width,
+                               id_='type')
 
             sub_comps = [FormComponent([enabled_opt, mode, type_, install_mode, uninstall_mode, upgrade_mode, downgrade_mode], spaces=False)]
             return TabComponent(self.i18n['core.config.tab.backup'].capitalize(), PanelComponent(sub_comps), None, 'core.bkp')
-
-    def _gen_select(self, label: str, tip: str, id_: str, opts: List[tuple], value: object, max_width: int, type_: SelectViewType = SelectViewType.RADIO):
-        inp_opts = [InputOption(label=o[0].capitalize(), value=o[1], tooltip=o[2]) for o in opts]
-        def_opt = [o for o in inp_opts if o.value == value]
-        return SingleSelectComponent(label=label,
-                                     tooltip=tip,
-                                     options=inp_opts,
-                                     default_option=def_opt[0] if def_opt else inp_opts[0],
-                                     max_per_line=len(inp_opts),
-                                     max_width=max_width,
-                                     type_=type_,
-                                     id_=id_)
-
