@@ -5,14 +5,16 @@ import traceback
 from datetime import datetime, timedelta
 from io import StringIO
 from pathlib import Path
-from typing import List, Type, Set, Tuple
+from typing import List, Type, Set, Tuple, Optional
 
 import requests
-from PyQt5.QtCore import QThread, pyqtSignal
+from PyQt5.QtCore import QThread, pyqtSignal, QObject
+from PyQt5.QtGui import QIcon
+from PyQt5.QtWidgets import QWidget
 
 from bauh import LOGS_PATH
 from bauh.api.abstract.cache import MemoryCache
-from bauh.api.abstract.controller import SoftwareManager, UpgradeRequirement, UpgradeRequirements
+from bauh.api.abstract.controller import SoftwareManager, UpgradeRequirement, UpgradeRequirements, SoftwareAction
 from bauh.api.abstract.handler import ProcessWatcher
 from bauh.api.abstract.model import PackageStatus, SoftwarePackage, CustomSoftwareAction
 from bauh.api.abstract.view import MessageType, MultipleSelectComponent, InputOption, TextComponent, \
@@ -20,14 +22,15 @@ from bauh.api.abstract.view import MessageType, MultipleSelectComponent, InputOp
 from bauh.api.exception import NoInternetException
 from bauh.commons import user
 from bauh.commons.html import bold
+from bauh.commons.internet import InternetChecker
 from bauh.commons.system import get_human_size_str, ProcessHandler, SimpleProcess
 from bauh.view.core import timeshift
-from bauh.view.core.config import read_config
+from bauh.view.core.config import CoreConfigManager
 from bauh.view.qt import commons
 from bauh.view.qt.view_model import PackageView, PackageViewStatus
 from bauh.view.util.translation import I18n
 
-RE_VERSION_IN_NAME = re.compile(r'\s+version\s+[\w\.]+\s*$')
+RE_VERSION_IN_NAME = re.compile(r'\s+version\s+[\w.]+\s*$')
 
 
 class AsyncAction(QThread, ProcessWatcher):
@@ -42,21 +45,26 @@ class AsyncAction(QThread, ProcessWatcher):
     signal_root_password = pyqtSignal()
     signal_progress_control = pyqtSignal(bool)
 
-    def __init__(self):
+    def __init__(self, root_password: Optional[Tuple[bool, str]] = None):
         super(AsyncAction, self).__init__()
         self.wait_confirmation = False
         self.confirmation_res = None
-        self.root_password = None
+        self.root_password = root_password
         self.stop = False
 
     def request_confirmation(self, title: str, body: str, components: List[ViewComponent] = None,
-                             confirmation_label: str = None, deny_label: str = None, deny_button: bool = True, window_cancel: bool = False) -> bool:
+                             confirmation_label: str = None, deny_label: str = None, deny_button: bool = True,
+                             window_cancel: bool = False,
+                             confirmation_button: bool = True) -> bool:
         self.wait_confirmation = True
-        self.signal_confirmation.emit({'title': title, 'body': body, 'components': components, 'confirmation_label': confirmation_label, 'deny_label': deny_label, 'deny_button': deny_button, 'window_cancel': window_cancel})
+        self.signal_confirmation.emit({'title': title, 'body': body, 'components': components,
+                                       'confirmation_label': confirmation_label, 'deny_label': deny_label,
+                                       'deny_button': deny_button, 'window_cancel': window_cancel,
+                                       'confirmation_button': confirmation_button})
         self.wait_user()
         return self.confirmation_res
 
-    def request_root_password(self) -> Tuple[str, bool]:
+    def request_root_password(self) -> Tuple[bool, Optional[str]]:
         self.wait_confirmation = True
         self.signal_root_password.emit()
         self.wait_user()
@@ -68,8 +76,8 @@ class AsyncAction(QThread, ProcessWatcher):
         self.confirmation_res = res
         self.wait_confirmation = False
 
-    def set_root_password(self, password: str, valid: bool):
-        self.root_password = (password, valid)
+    def set_root_password(self, valid: bool, password: str):
+        self.root_password = (valid, password)
         self.wait_confirmation = False
 
     def wait_user(self):
@@ -116,64 +124,78 @@ class AsyncAction(QThread, ProcessWatcher):
 
         return False
 
-    def _generate_backup(self, app_config: dict, i18n: I18n, root_password: str) -> bool:
-        if timeshift.is_available():
-            if app_config['backup']['mode'] not in ('only_one', 'incremental'):
-                self.show_message(title=self.i18n['error'].capitalize(),
-                                  body='{}: {}'.format(self.i18n['action.backup.invalid_mode'], bold(app_config['backup']['mode'])),
-                                  type_=MessageType.ERROR)
-                return False
+    def _generate_backup(self, app_config: dict, i18n: I18n, root_password: Optional[str]) -> bool:
+        if app_config['backup']['mode'] not in ('only_one', 'incremental'):
+            self.show_message(title=self.i18n['error'].capitalize(),
+                              body='{}: {}'.format(self.i18n['action.backup.invalid_mode'],bold(app_config['backup']['mode'])),
+                              type_=MessageType.ERROR)
+            self.change_substatus('')
+            return False
 
-            if not user.is_root() and not root_password:
-                root_pwd, valid = self.request_root_password()
-            else:
-                root_pwd, valid = root_password, True
+        handler = ProcessHandler(self)
+        if app_config['backup']['mode'] == 'only_one':
+            self.change_substatus('[{}] {}'.format(i18n['core.config.tab.backup'].lower(), i18n['action.backup.substatus.delete']))
+            deleted, _ = handler.handle_simple(timeshift.delete_all_snapshots(root_password))
 
-            if not valid:
-                return False
-
-            handler = ProcessHandler(self)
-            if app_config['backup']['mode'] == 'only_one':
-                self.change_substatus('[{}] {}'.format(i18n['core.config.tab.backup'].lower(), i18n['action.backup.substatus.delete']))
-                deleted, _ = handler.handle_simple(timeshift.delete_all_snapshots(root_pwd))
-
-                if not deleted and not self.request_confirmation(title=i18n['core.config.tab.backup'],
-                                                                 body='{}. {}'.format(i18n['action.backup.error.delete'], i18n['action.backup.error.proceed']),
-                                                                 confirmation_label=i18n['yes'].capitalize(),
-                                                                 deny_label=i18n['no'].capitalize()):
-                    return False
-
-            self.change_substatus('[{}] {}'.format(i18n['core.config.tab.backup'].lower(), i18n['action.backup.substatus.create']))
-            created, _ = handler.handle_simple(timeshift.create_snapshot(root_pwd, app_config['backup']['type']))
-
-            if not created and not self.request_confirmation(title=i18n['core.config.tab.backup'],
-                                                             body='{}. {}'.format(i18n['action.backup.error.create'],
+            if not deleted and not self.request_confirmation(title=i18n['core.config.tab.backup'],
+                                                             body='{}. {}'.format(i18n['action.backup.error.delete'],
                                                                                   i18n['action.backup.error.proceed']),
                                                              confirmation_label=i18n['yes'].capitalize(),
                                                              deny_label=i18n['no'].capitalize()):
+                self.change_substatus('')
                 return False
 
+        self.change_substatus('[{}] {}'.format(i18n['core.config.tab.backup'].lower(), i18n['action.backup.substatus.create']))
+        created, _ = handler.handle_simple(timeshift.create_snapshot(root_password, app_config['backup']['type']))
+
+        if not created and not self.request_confirmation(title=i18n['core.config.tab.backup'],
+                                                         body='{}. {}'.format(i18n['action.backup.error.create'],
+                                                                              i18n['action.backup.error.proceed']),
+                                                         confirmation_label=i18n['yes'].capitalize(),
+                                                         deny_label=i18n['no'].capitalize()):
+            self.change_substatus('')
+            return False
+
+        self.change_substatus('')
         return True
 
-    def request_backup(self, app_config: dict, key: str, i18n: I18n, root_password: str = None) -> bool:
-        if bool(app_config['backup']['enabled']) and timeshift.is_available():
-            val = app_config['backup'][key] if key else None
+    def _check_backup_requirements(self, app_config: dict, pkg: Optional[PackageView], action_key: Optional[str]) -> bool:
+        if pkg and not pkg.model.supports_backup():
+            return False
 
-            if val is None:  # ask mode
-                if self.request_confirmation(title=i18n['core.config.tab.backup'],
+        if action_key and app_config['backup'][action_key] is False:
+            return False
+
+        return bool(app_config['backup']['enabled']) and timeshift.is_available()
+
+    def _should_backup(self, action_key: str, app_config: dict, i18n: I18n) -> bool:
+        # backup -> true: do not ask, only execute | false: do not ask or execute | None: ask
+        backup = app_config['backup'][action_key] if action_key else None
+
+        if backup is None:
+            return self.request_confirmation(title=i18n['core.config.tab.backup'],
                                              body=i18n['action.backup.msg'],
                                              confirmation_label=i18n['yes'].capitalize(),
-                                             deny_label=i18n['no'].capitalize()):
-                    res = self._generate_backup(app_config, i18n, root_password)
-                    self.change_substatus('')
-                    return res
+                                             deny_label=i18n['no'].capitalize())
+        else:
+            return backup
 
-            elif val is True:  # direct mode
-                res = self._generate_backup(app_config, i18n, root_password)
-                self.change_substatus('')
-                return res
+    def request_backup(self, action_key: Optional[str], pkg: Optional[PackageView], i18n: I18n, app_config: dict, root_password: Optional[str], backup_only: bool = False) -> Tuple[bool, Optional[str]]:
+        if not backup_only:
+            if not self._check_backup_requirements(app_config=app_config, pkg=pkg, action_key=action_key):
+                return True, root_password
 
-        return True
+            if not self._should_backup(action_key=action_key, app_config=app_config, i18n=i18n):
+                return True, root_password
+
+        pwd = root_password
+        if not user.is_root() and pwd is None:
+            valid_password, pwd = self.request_root_password()
+
+            if not valid_password:
+                return False, None
+
+        return self._generate_backup(app_config=app_config, i18n=i18n, root_password=pwd), pwd
 
 
 class UpgradeSelected(AsyncAction):
@@ -181,31 +203,38 @@ class UpgradeSelected(AsyncAction):
     LOGS_DIR = '{}/upgrade'.format(LOGS_PATH)
     SUMMARY_FILE = LOGS_DIR + '/{}_summary.txt'
 
-    def __init__(self, manager: SoftwareManager, i18n: I18n, pkgs: List[PackageView] = None):
+    def __init__(self, manager: SoftwareManager, internet_checker: InternetChecker, i18n: I18n, pkgs: List[PackageView] = None):
         super(UpgradeSelected, self).__init__()
         self.pkgs = pkgs
         self.manager = manager
         self.i18n = i18n
+        self.internet_checker = internet_checker
 
-    def _req_as_option(self, req: UpgradeRequirement, tooltip: bool = True, custom_tooltip: str = None) -> InputOption:
+    def _req_as_option(self, req: UpgradeRequirement, tooltip: bool = True, custom_tooltip: str = None, required_size: bool = True, display_sizes: bool = True) -> InputOption:
         if req.pkg.installed:
             icon_path = req.pkg.get_disk_icon_path()
 
-            if not icon_path or not os.path.isfile(icon_path):
+            if not icon_path:
+                icon_path = req.pkg.get_type_icon_path()
+            elif not os.path.isfile(icon_path) and QIcon.fromTheme(icon_path).isNull():
                 icon_path = req.pkg.get_type_icon_path()
 
         else:
             icon_path = req.pkg.get_type_icon_path()
 
-        size_str = '{}: {}'.format(self.i18n['size'].capitalize(),
-                                   '?' if req.extra_size is None else get_human_size_str(req.extra_size))
-        if req.extra_size != req.required_size:
-            size_str += ' ( {}: {} )'.format(self.i18n['action.update.pkg.required_size'].capitalize(),
-                                             '?' if req.required_size is None else get_human_size_str(req.required_size))
+        size_str = None
+        if display_sizes:
+            size_str = '{}: {}'.format(self.i18n['size'].capitalize(),
+                                       '?' if req.extra_size is None else get_human_size_str(req.extra_size))
+            if required_size and req.extra_size != req.required_size:
+                size_str += ' ( {}: {} )'.format(self.i18n['action.update.pkg.required_size'].capitalize(),
+                                                 '?' if req.required_size is None else get_human_size_str(req.required_size))
 
-        label = '{}{} - {}'.format(req.pkg.name,
-                                   ' ( {} )'.format(req.pkg.latest_version) if req.pkg.latest_version else '',
-                                   size_str)
+        label = '{}{}'.format(req.pkg.name,
+                              ' ( {} )'.format(req.pkg.latest_version) if req.pkg.latest_version else '')
+
+        if size_str:
+            label += ' - {}'.format(size_str)
 
         return InputOption(label=label,
                            value=None,
@@ -224,14 +253,17 @@ class UpgradeSelected(AsyncAction):
 
         return required, extra
 
-    def _gen_cannot_update_form(self, reqs: List[UpgradeRequirement]) -> FormComponent:
-        opts = [self._req_as_option(r, False, r.reason) for r in reqs]
+    def _gen_cannot_upgrade_form(self, reqs: List[UpgradeRequirement]) -> FormComponent:
+        reqs.sort(key=UpgradeRequirement.sort_by_priority)
+        opts = [self._req_as_option(req=r, tooltip=False, custom_tooltip=r.reason, display_sizes=False) for r in reqs]
         comps = [MultipleSelectComponent(label='', options=opts, default_options=set(opts))]
 
-        return FormComponent(label=self.i18n['action.update.cannot_update_label'], components=comps)
+        return FormComponent(label='{} ( {}: {} )'.format(self.i18n['action.update.cannot_update_label'],
+                                                          self.i18n['amount'].capitalize(), len(opts)),
+                             components=comps)
 
     def _gen_to_install_form(self, reqs: List[UpgradeRequirement]) -> Tuple[FormComponent, Tuple[int, int]]:
-        opts = [self._req_as_option(r) for r in reqs]
+        opts = [self._req_as_option(r, custom_tooltip=r.reason) for r in reqs]
         comps = [MultipleSelectComponent(label='', options=opts, default_options=set(opts))]
         required_size, extra_size = self._sum_pkgs_size(reqs)
 
@@ -245,7 +277,7 @@ class UpgradeSelected(AsyncAction):
         return FormComponent(label=lb, components=comps), (required_size, extra_size)
 
     def _gen_to_remove_form(self, reqs: List[UpgradeRequirement]) -> FormComponent:
-        opts = [self._req_as_option(r, False, r.reason) for r in reqs]
+        opts = [self._req_as_option(r, False, r.reason, required_size=False) for r in reqs]
         comps = [MultipleSelectComponent(label='', options=opts, default_options=set(opts))]
         required_size, extra_size = self._sum_pkgs_size(reqs)
 
@@ -271,9 +303,9 @@ class UpgradeSelected(AsyncAction):
 
         return FormComponent(label=lb, components=comps), (required_size, extra_size)
 
-    def _request_password(self) -> Tuple[bool, str]:
+    def _request_password(self) -> Tuple[bool, Optional[str]]:
         if not user.is_root():
-            pwd, success = self.request_root_password()
+            success, pwd = self.request_root_password()
 
             if not success:
                 return False, None
@@ -356,16 +388,38 @@ class UpgradeSelected(AsyncAction):
         except:
             traceback.print_exc()
 
+    def _handle_internet_off(self):
+        self.pkgs = None
+        self.print(self.i18n['internet.required'])
+        self.notify_finished({'success': False, 'updated': 0, 'types': set(), 'id': None})
+
     def run(self):
-        valid_password, root_password = self._request_password()
+        if not self.internet_checker.is_available():
+            return self._handle_internet_off()
+
+        root_user = user.is_root()
+        to_update, upgrade_requires_root, bkp_supported = [], False, False
+
+        for pkg in self.pkgs:
+            if pkg.model.update and not pkg.model.is_update_ignored() and pkg.update_checked:
+                to_update.append(pkg)
+
+                if not bkp_supported and pkg.model.supports_backup():
+                    bkp_supported = True
+
+                if not root_user and not upgrade_requires_root and self.manager.requires_root(SoftwareAction.UPGRADE, pkg.model):
+                    upgrade_requires_root = True
+
+        if upgrade_requires_root:
+            valid_password, root_password = self._request_password()
+        else:
+            valid_password, root_password = True, None
 
         if not valid_password:
             self.notify_finished({'success': False, 'updated': 0, 'types': set(), 'id': None})
             self.pkgs = None
             return
 
-        to_update = [pkg for pkg in self.pkgs if pkg.model.update and not pkg.model.is_update_ignored() and pkg.update_checked]
-
         if len(to_update) > 1:
             self.disable_progress_controll()
         else:
@@ -388,7 +442,7 @@ class UpgradeSelected(AsyncAction):
         comps, required_size, extra_size = [], 0, 0
 
         if requirements.cannot_upgrade:
-            comps.append(self._gen_cannot_update_form(requirements.cannot_upgrade))
+            comps.append(self._gen_cannot_upgrade_form(requirements.cannot_upgrade))
 
         if requirements.to_install:
             req_form, reqs_size = self._gen_to_install_form(requirements.to_install)
@@ -399,10 +453,13 @@ class UpgradeSelected(AsyncAction):
         if requirements.to_remove:
             comps.append(self._gen_to_remove_form(requirements.to_remove))
 
-        updates_form, updates_size = self._gen_to_update_form(requirements.to_upgrade)
-        required_size += updates_size[0]
-        extra_size += updates_size[1]
-        comps.append(updates_form)
+        can_upgrade = False
+        if requirements.to_upgrade:
+            can_upgrade = True
+            updates_form, updates_size = self._gen_to_update_form(requirements.to_upgrade)
+            required_size += updates_size[0]
+            extra_size += updates_size[1]
+            comps.append(updates_form)
 
         extra_size_text = '{}: {}'.format(self.i18n['action.update.total_size'].capitalize(), get_human_size_str(extra_size))
         req_size_text = '{}: {}'.format(self.i18n['action.update.required_size'].capitalize(),
@@ -411,35 +468,50 @@ class UpgradeSelected(AsyncAction):
         comps.insert(1, TextComponent(''))
 
         if not self.request_confirmation(title=self.i18n['action.update.summary'].capitalize(), body='', components=comps,
-                                         confirmation_label=self.i18n['proceed'].capitalize(), deny_label=self.i18n['cancel'].capitalize()):
+                                         confirmation_label=self.i18n['proceed'].capitalize(), deny_label=self.i18n['cancel'].capitalize(),
+                                         confirmation_button=can_upgrade):
             self.notify_finished({'success': success, 'updated': updated, 'types': updated_types, 'id': None})
             self.pkgs = None
             return
 
+        if not self.internet_checker.is_available():
+            return self._handle_internet_off()
+
         self.change_substatus('')
 
-        app_config = read_config()
+        app_config = CoreConfigManager().get_config()
 
-        # trim dialog
+        # backup dialog ( if enabled, supported and accepted )
+        should_backup = bkp_supported
+        should_backup = should_backup and self._check_backup_requirements(app_config=app_config, pkg=None, action_key='upgrade')
+        should_backup = should_backup and self._should_backup(action_key='upgrade', app_config=app_config, i18n=self.i18n)
+
+        # trim dialog ( if enabled and accepted )
         if app_config['disk']['trim']['after_upgrade'] is not False:
             should_trim = app_config['disk']['trim']['after_upgrade'] or self._ask_for_trim()
         else:
             should_trim = False
 
-        # backup process ( if enabled, supported and accepted )
-        if bool(app_config['backup']['enabled']) and app_config['backup']['upgrade'] in (True, None) and timeshift.is_available():
-            any_requires_bkp = False
+        if should_trim and not root_user and root_password is None:  # trim requires root and the password might have not being asked yet
+            valid_password, root_password = self._request_password()
 
-            for dep in requirements.to_upgrade:
-                if dep.pkg.supports_backup():
-                    any_requires_bkp = True
-                    break
+            if not valid_password:
+                self.notify_finished({'success': False, 'updated': 0, 'types': set(), 'id': None})
+                self.pkgs = None
+                return
 
-            if any_requires_bkp:
-                if not self.request_backup(app_config, 'upgrade', self.i18n, root_password):
-                    self.notify_finished({'success': success, 'updated': updated, 'types': updated_types, 'id': None})
-                    self.pkgs = None
-                    return
+        # performing backup
+        if should_backup:
+            proceed, root_password = self.request_backup(action_key='upgrade',
+                                                         app_config=app_config,
+                                                         i18n=self.i18n,
+                                                         root_password=root_password,
+                                                         pkg=None,
+                                                         backup_only=True)
+            if not proceed:
+                self.notify_finished({'success': False, 'updated': 0, 'types': set(), 'id': None})
+                self.pkgs = None
+                return
 
         self.change_substatus('')
 
@@ -477,7 +549,8 @@ class RefreshApps(AsyncAction):
 
     def run(self):
         try:
-            res = self.manager.read_installed(pkg_types=self.pkg_types)
+            res = self.manager.read_installed(pkg_types=self.pkg_types, disk_loader=None, only_apps=False,
+                                              internet_available=True, limit=-1)
             refreshed_types = set()
 
             if res:
@@ -508,12 +581,16 @@ class UninstallPackage(AsyncAction):
 
     def run(self):
         if self.pkg:
-            if self.pkg.model.supports_backup():
-                if not self.request_backup(read_config(), 'uninstall', self.i18n, self.root_pwd):
-                    self.notify_finished(False)
-                    self.pkg = None
-                    self.root_pwd = None
-                    return
+            proceed, _ = self.request_backup(action_key='uninstall',
+                                             pkg=self.pkg,
+                                             i18n=self.i18n,
+                                             root_password=self.root_pwd,
+                                             app_config=CoreConfigManager().get_config())
+            if not proceed:
+                self.notify_finished({'success': False, 'removed': None, 'pkg': self.pkg})
+                self.pkg = None
+                self.root_pwd = None
+                return
 
             try:
                 res = self.manager.uninstall(self.pkg.model, self.root_pwd, self, None)
@@ -547,12 +624,17 @@ class DowngradePackage(AsyncAction):
         if self.pkg:
             success = False
 
-            if self.pkg.model.supports_backup():
-                if not self.request_backup(read_config(), 'downgrade', self.i18n, self.root_pwd):
-                    self.notify_finished({'app': self.pkg, 'success': success})
-                    self.pkg = None
-                    self.root_pwd = None
-                    return
+            proceed, _ = self.request_backup(action_key='downgrade',
+                                             pkg=self.pkg,
+                                             i18n=self.i18n,
+                                             root_password=self.root_pwd,
+                                             app_config=CoreConfigManager().get_config())
+
+            if not proceed:
+                self.notify_finished({'app': self.pkg, 'success': success})
+                self.pkg = None
+                self.root_pwd = None
+                return
 
             try:
                 success = self.manager.downgrade(self.pkg.model, self.root_pwd, self)
@@ -575,7 +657,12 @@ class ShowPackageInfo(AsyncAction):
     def run(self):
         if self.pkg:
             info = {'__app__': self.pkg}
-            info.update(self.manager.get_info(self.pkg.model))
+
+            pkg_info = self.manager.get_info(self.pkg.model)
+
+            if pkg_info:
+                info.update(pkg_info)
+
             self.notify_finished(info)
             self.pkg = None
 
@@ -610,7 +697,7 @@ class SearchPackages(AsyncAction):
 
         if self.word:
             try:
-                res = self.manager.search(self.word)
+                res = self.manager.search(words=self.word, disk_loader=None, limit=-1, is_url=False)
                 search_res['pkgs_found'].extend(res.installed)
                 search_res['pkgs_found'].extend(res.new)
             except NoInternetException:
@@ -634,10 +721,17 @@ class InstallPackage(AsyncAction):
         if self.pkg:
             res = {'success': False, 'installed': None, 'removed': None, 'pkg': self.pkg}
 
-            if self.pkg.model.supports_backup():
-                if not self.request_backup(read_config(), 'install', self.i18n, self.root_pwd):
-                    self.signal_finished.emit(res)
-                    return
+            proceed, _ = self.request_backup(action_key='install',
+                                             pkg=self.pkg,
+                                             i18n=self.i18n,
+                                             root_password=self.root_pwd,
+                                             app_config=CoreConfigManager().get_config())
+
+            if not proceed:
+                self.signal_finished.emit(res)
+                self.pkg = None
+                self.root_pwd = None
+                return
 
             try:
                 transaction_res = self.manager.install(self.pkg.model, self.root_pwd, None, self)
@@ -814,7 +908,7 @@ class ListWarnings(QThread):
         self.man = man
 
     def run(self):
-        warnings = self.man.list_warnings()
+        warnings = self.man.list_warnings(internet_available=True)
         if warnings:
             self.signal_warnings.emit(warnings)
 
@@ -928,25 +1022,33 @@ class CustomAction(AsyncAction):
         self.i18n = i18n
 
     def run(self):
+        res = {'success': False, 'pkg': self.pkg, 'action': self.custom_action, 'error': None, 'error_type': MessageType.ERROR}
+
         if self.custom_action.backup:
-            app_config = read_config()
-            if not self.request_backup(app_config, None, self.i18n, self.root_pwd):
-                self.notify_finished({'success': False, 'pkg': self.pkg, 'action': self.custom_action})
+            proceed, _ = self.request_backup(app_config=CoreConfigManager.get_config(),
+                                             action_key=None,
+                                             i18n=self.i18n,
+                                             root_password=self.root_pwd,
+                                             pkg=self.pkg)
+            if not proceed:
+                self.notify_finished(res)
                 self.pkg = None
                 self.custom_action = None
                 self.root_pwd = None
                 return
 
         try:
-            success = self.manager.execute_custom_action(action=self.custom_action,
-                                                         pkg=self.pkg.model if self.pkg else None,
-                                                         root_password=self.root_pwd,
-                                                         watcher=self)
+            res['success'] = self.manager.execute_custom_action(action=self.custom_action,
+                                                                pkg=self.pkg.model if self.pkg else None,
+                                                                root_password=self.root_pwd,
+                                                                watcher=self)
         except (requests.exceptions.ConnectionError, NoInternetException):
-            success = False
+            res['success'] = False
+            res['error'] = 'internet.required'
+            res['error_type'] = MessageType.WARNING
             self.signal_output.emit(self.i18n['internet.required'])
 
-        self.notify_finished({'success': success, 'pkg': self.pkg, 'action': self.custom_action})
+        self.notify_finished(res)
         self.pkg = None
         self.custom_action = None
         self.root_pwd = None
@@ -987,3 +1089,36 @@ class IgnorePackageUpdates(AsyncAction):
 
             finally:
                 self.pkg = None
+
+
+class SaveTheme(QThread):
+
+    def __init__(self, theme_key: Optional[str], parent: Optional[QWidget] = None):
+        super(SaveTheme, self).__init__(parent=parent)
+        self.theme_key = theme_key
+
+    def run(self):
+        if self.theme_key:
+            configman = CoreConfigManager()
+            core_config = configman.get_config()
+            core_config['ui']['theme'] = self.theme_key
+
+            try:
+                configman.save_config(core_config)
+            except:
+                traceback.print_exc()
+
+
+class StartAsyncAction(QThread):
+
+    signal_start = pyqtSignal()
+
+    def __init__(self, delay_in_milis: int = -1, parent: Optional[QObject] = None):
+        super(StartAsyncAction, self).__init__(parent=parent)
+        self.delay = delay_in_milis
+
+    def run(self):
+        if self.delay > 0:
+            self.msleep(self.delay)
+
+        self.signal_start.emit()
